'******************************************************************************
'  (c) Jan Petersen
'      Orevej 6
'      DK 4760 Vordingborg
'      jan_p@post8.tele.dk
'------------------------------------------------------------------------------
'  Date:          24.02.2000
'  Compiler:      Bascom AVR 1.0.8b
'  File:          Serial2Gpib.bas
'  Description:   RS232 to GPIB converter  +
'                 HPGL to GDU converter
'
'  This program is made specific for my Tektronix 4662 flatbed plotter
'  with GPIB (IEE488) interface, and a strange GDU coordinate system..!
'  Now it communicate whit a HPGL device-driver trough a normal
'  RS 232 serialport. Hey man...
'------------------------------------------------------------------------------
'
'  Input:
'  RS232      :19200N81R
'  Handshake  :RTS/CTS  specific for (Eagle CAM processor)
'
'  Output:
'  GPIB       :DATA[0..7] (Inverted !Activ Low!)
'  Handshake  :DAV        (DataValid)
'  Handshake  :NDAC       (NoDataAccepted)
'  Handshake  :NRFD       (NotReadyForData)
'------------------------------------------------------------------------------
'
'  HPGL commands:                 Tektronix 4662 Plotter Commands:
'  PU;          (PenUp)           M[cr]
'  PD;          (PenDown)         D[cr]
'  PA xxx,yyy;  (PlotAbsolut)     Dxxx,yyy[cr]
'  VS xx;       (VelocitySpeed)   BYxx[cr]
'  SP 0;        (SelectPen)       End Of File so goto home = M150,100[cr]
'
'  GDU = GraphicDataUnit
'  HPGL = HeweletPackardGraphicLanguage
'
'  1 GDU = 2.54 mm
'  1 HPGL unit = 0.025 mm
'
'  100 HPGL units = 1    GDU
'    1 HPGL units = 0.01 GDU
'
'  All coordinates are divided by 100
'
'  A typical input format would look like this (Eagle Cam processor-HPGL driver)
'
'  IN; IP 0,0,100,100; SC 0,100,0,100;
'  VS 120
'  SP 2;
'  PU;
'  PA 19,18;
'  PD;
'  PA 1035,18;
'  PU; SP 0; PA 0,0;
'
'  Converted output to Tektronix plotter
'
'  BY120
'  M0.19,0.18
'  D10.35,0.18
'  M150.100
'******************************************************************************

$regfile = "2313def.dat"
$crystal = 7372800                                  '7.372800 MHz crystal
$baud = 19200                                       '@ 19.2 Kbaud error is 0.0%

Dim Result As String * 15
Dim Value_x As String * 7
Dim Value_y As String * 7
Dim Low_ As String * 2
Dim High_ As String * 3
Dim Command As String * 1
Dim A As Byte , B As Byte
Dim H As Byte , L As Byte , Data_inv As Byte

' Init ports ---------------------------------------
Ddrd = &B00010100
Config Portb = Output

Data_out Alias Portb                                'Data byte to GPIB PortB
                                                    'RX                PortD.0
                                                    'TX                PortD.1
Rts_cts Alias Portd.2                               'RTS/CTS handshake PortD.2
Dav Alias Portd.4                                   'DAV               PortD.4
                                                    'NDAC              PortD.5
                                                    'NRFD              PortD.6

' Start --------------------------------------------
set dav                                             'No DataValid on GPIB port
Command = "M"                                       'set Command to MOVE
Gosub _reset                                        'reset variable
Reset Rts_cts                                       'uP ready

' Main loop ----------------------------------------
Do
  A = Waitkey()                                     'wait for a character
  set Rts_cts                                       'uP NOT ready

  Select Case A
    Case 10 : Gosub _reset                          'linefeed
    Case 13 : Gosub _reset                          'carriage return
    Case 32 : Gosub _space                          'space
    Case 44 : Gosub _comma                          'comma
    Case 59 : Gosub _semicolon                      'semicolon
    Case Else : Result = Result + Chr(a)            'add the character to string
  End Select

  Reset Rts_cts                                     'uP ready
Loop


_space:
' SPACE
'***************************************************
  Select Case Result
    Case "VS" : Command = "V"                       'VelocitySpeed
    Case "SP" : Command = "S"                       'SelectPen
    Case "PU" : Command = "M"                       'PenUp
    Case "PD" : Command = "D"                       'PenDown
  End Select

  Gosub _reset
Return


_comma:
' COMMA
'***************************************************
  Value_x = Result                                  'value_x = X-coordinate
  Result = ""                                       'clear Result
Return


_semicolon:
' SEMICOLON
'***************************************************
  Select Case Command
    Case "V" : Gosub _vs                            'Velocity
    Case "S" : Gosub _sp                            'Select Pen
    Case "R" : Gosub _reset                         '/Dummy Chr...!
  End Select                                        '/SP 0 command was detected
                                                    '/in the _sp SUB
                                                    '/means ..EOF..
  Select Case Result
    Case "PU" : Command = "M"                       'Move
    Case "PD" : Command = "D"                       'Draw
    Case Else : Gosub _make_string                  'value_y is in result
  End Select

  Gosub _reset
Return


_vs:
' PEN SPEED  (velocity speed)
'***************************************************
  If Result <> "0" Then
    Result = "BY" + Result + Chr(13)
    Gosub _gpib_out                                 'BYxx[cr]
  End If

  Gosub _reset : Command = "M"
Return


_sp:
' SELECT PEN
' if a "SP 0" command is recived
' it means "MOVE HOME" and EOF
'***************************************************
  If Result = "0" Then
    Result = "M150," + "100" + Chr(13)
    Gosub _gpib_out                                 'M150,100[cr]
    Command = "R"                                   '/R means EOF
                                                    '/and is at dummy Chr
                                                    '/used in _semicolon SUB
  Else
    Command = "M"
  End If

  Gosub _reset
Return


_make_string:
' Prepare the string to send to GPIB
'***************************************************
  If Result = "" Then Return

  If Value_x = "" Then
    Gosub _reset                                    '/happens on the first line
    Return                                          '/when value_x is NUL
  End If

  Gosub _divide                                     'value_Y is in result
  Value_y = Result                                  'value_Y = value_y / 100

  Result = Value_x                                  'now value_X is in result

  Gosub _divide
  Value_x = Result                                  'value_X = value_x / 100

' Assemble the string  -----------------------------
  Result = Command + Value_x + "," + Value_y + Chr(13)

  Gosub _gpib_out                                   'eg.Result = "M12.78,56[cr]"
  Gosub _reset
Return


_reset:
' Reset some variable
' This SUB saves lot of ROM Space.. :o)
'***************************************************
  Result = ""                                       'saves some program memory
  Value_x = ""                                      'when variables are cleared
  Value_y = ""                                      'in one SUB
Return


_divide:
' divide by 100
' SINGLE aritmic don't work yet, so we do It like this..!
'***************************************************
  High_ = "" : Low_ = ""

  If Result = "0" Then Return                       'don't divide, value is zero

  L = Len(result)                                   'length of string

  If L > 2 Then H = L - 2

  If L = 1 Then Low_ = Right(result , 1)            '1'st decimal value
  If L > 1 Then Low_ = Right(result , 2)            '2'nd decimal value
  If L > 2 Then High_ = Mid(result , 1 , H)         'real value

  If L = 1 Then Result = "0.0" + Low_               '1 decimal
  If L = 2 Then Result = "0." + Low_                '2 decimal
  If L > 2 Then Result = High_ + "." + Low_         'real + decimal

  B = Val(low_)
  If B = 0 Then Result = High_                      'ONLY real
Return


_gpib_out:
' Output data to GPIB port
' Data is inverted
' NRFS,NDAC and DAV is handshake
'***************************************************
  L = Len(result)                                   'how long is the string..?

  For H = 1 To L                                    'loop the string
    Low_ = ""                                       're use a old variable

    Low_ = Mid(result , H , 1)                      'pick out the character
    Data_inv = 255 - Asc(low_)                      'invert the character

    Bitwait Pind.6 , Set                            'wait until (NRFD) goes HIGH

    Data_out = Data_inv                             'Set the data on the port

    Reset Dav                                       'DataValid on port

    Bitwait Portd.6 , Reset                         'wait until (NRFD) goes LOW

    Bitwait Pind.5 , Set                            'wait until (NDAC) goes HIGH

    set dav                                         'Data NOT Valid on port

    Bitwait Portd.5 , Reset                         'wait until (NDAC) goes LOW
  Next H

  Bitwait Pind.6 , Set                              'wait until (NRFD) goes HIGH
Return

End